/*
 * reiserfs_err.h:
 * This file is automatically generated; please do not edit it.
 */

#include <et/com_err.h>

#define REISERFS_ET_BREAD_FAILED                 (50094765501195648L)
#define REISERFS_ET_BAD_MAGIC                    (50094765501195647L)
#define REISERFS_ET_BAD_SUPER                    (50094765501195646L)
#define REISERFS_ET_SMALL_PARTITION              (50094765501195645L)
#define REISERFS_ET_NOT_ENOUGH_BLOCKS            (50094765501195644L)
#define REISERFS_ET_TOO_SMALL                    (50094765501195643L)
#define REISERFS_ET_GETBLK_FAILED                (50094765501195642L)
extern const struct error_table et_reiserfs_error_table;
extern void initialize_reiserfs_error_table(void);

/* For compatibility with Heimdal */
extern void initialize_reiserfs_error_table_r(struct et_list **list);

#define ERROR_TABLE_BASE_reiserfs (50094765501195648L)

/* for compatibility with older versions... */
#define init_reiserfs_err_tbl initialize_reiserfs_error_table
#define reiserfs_err_base ERROR_TABLE_BASE_reiserfs
